import {
  IDictionary, I18n, IEventParam, Metadata, Loader, Config,
} from '@zeedhi/core';
import {
  Text, LoadingService, DialogService, Tabs, Tab, Button, Card, Badge,
} from '@zeedhi/common';
import { ScmMask } from '@/config';
import { RequestErrorLog } from '@/server/request-logger';

export class ErrorsController {
  private utilities = Loader.getInstance('Utilities');

  public showDetailError = false;

  public errorDescription = '';

  public errorDetail = '';

  public errorDataRow: IDictionary = [];

  public get errorsLogData() {
    const item = localStorage.getItem('request-error-log') || '[]';
    const errorLogs: RequestErrorLog[] = JSON.parse(item);
    const sorted = errorLogs.sort((a:RequestErrorLog, b:RequestErrorLog) => (a.timestamp > b.timestamp ? -1 : 1));
    return sorted;
  }

  public set errorsLogData(value: IDictionary[]) {
    console.warn(value);
  }

  public setTimestampError({ component }: IEventParam<Text>) {
    const timestamp = component.name.split('_')[1];
    component.text = `${I18n.translate('ID')}: ${timestamp}`;
  }

  public async mountErrors() {
    LoadingService.show(' ');
    const tabsErrors = Metadata.getInstance<Tabs>('tabsErrors');
    const errorsData = this.errorsLogData;
    const grouped = errorsData.reduce(
      (result: any, currentValue: any) => {
        (result[currentValue.date] = result[currentValue.date] || []).push(currentValue);
        return result;
      }, {},
    );
    if (Object.entries(grouped).length > 0) {
      if (tabsErrors.tabs.length > 0) {
        await Promise.all(tabsErrors.tabs.map(async (tab: IDictionary) => {
          await tab.children.map(async (child: IDictionary) => {
            await tab.removeChild(child.name);
          });
        })).then(() => {
          tabsErrors.tabs = [];
          tabsErrors.tabs = Object.entries(grouped).map((row: IDictionary) => this.newTabs(row[0], row[1]));
        });
      } else tabsErrors.tabs = Object.entries(grouped).map((row: IDictionary) => this.newTabs(row[0], row[1]));
    } else {
      const noData = Metadata.getInstance<Text>('noDataText');
      noData.isVisible = true;
      tabsErrors.isVisible = false;
    }
    LoadingService.hide();
  }

  public newTabs(title: string, dataErrors: IDictionary) {
    return new Tab({
      name: `${title}`,
      tabTitle: `${title}`,
      children: [
        {
          name: `iterableCardsErrors-${title}`,
          component: 'ZdIterableComponentRender',
          cssStyle: 'margin-bottom: 50px;',
          datasource: {
            uniqueKey: 'timestamp',
            type: 'memory',
            data: dataErrors,
          },
          events: {
            onMounted: '{{ErrorsController.markErrorsSeen}}',
          },
          componentMetadata: {
            name: 'cardError_<<row.timestamp>>',
            component: 'ZdCard',
            cssClass: 'zd-my-2',
            cols: 12,
            hover: true,
            ripple: true,
            width: '100vh',
            events: {
              click: '{{ErrorsController.clickErrorCard}}',
            },
            children: [
              {
                name: 'title_<<row.timestamp>>',
                component: 'ZdText',
                text: '<<row.objectName>>',
                tag: 'h3',
                cssStyle: {
                  overflow: 'hidden',
                  'white-space': 'nowrap',
                  'text-overflow': 'ellipsis',
                },
              },
              {
                name: 'rowTitle-<<row.timestamp>>',
                justify: 'space-between',
                align: 'start',
                component: 'ZdRow',
                children: [
                  {
                    name: 'colTitle-<<row.KEY>>-<<row.CDPRODUTO>>',
                    component: 'ZdCol',
                    cols: 9,
                    children: [
                      {
                        name: 'timestamp_<<row.timestamp>>',
                        component: 'ZdText',
                        events: {
                          onMounted: '{{ErrorsController.setTimestampError}}',
                        },
                      },
                      {
                        name: 'operation_<<row.timestamp>>_<<row.operation>>',
                        component: 'ZdText',
                        events: {
                          onMounted: '{{ErrorsController.translateOperation}}',
                        },
                      },
                      {
                        name: 'url_<<row.timestamp>>',
                        component: 'ZdText',
                        text: 'URL: <<row.url>>',
                      },
                      {
                        name: 'error_<<row.timestamp>>',
                        component: 'ZdText',
                        text: 'Erro: <<row.description>>',
                      },
                      // {
                      //   name: 'tooltipRow_<<row.timestamp>>',
                      //   component: 'ZdTooltip',
                      //   bottom: true,
                      //   label: 'Dados: <<row.row>>',
                      //   children: [
                      //     {
                      //       name: 'row-<<row.timestamp>>',
                      //       component: 'ZdText',
                      //       text: '<<row.row>>',
                      //       cssStyle: {
                      //         overflow: 'hidden',
                      //         'white-space': 'nowrap',
                      //         'text-overflow': 'ellipsis',
                      //       },
                      //     },
                      //   ],
                      // },
                    ],
                  },
                  {
                    name: 'deleteBtn-<<row.timestamp>>',
                    component: 'ZdButton',
                    iconName: 'mdi-trash-can',
                    icon: true,
                    right: true,
                    small: true,
                    cols: 3,
                    events: {
                      click: '{{ErrorsController.clickDeleteError}}',
                    },
                  },
                ],
              },
            ],
          },
        },
      ],
    });
  }

  public clickDeleteError({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteError',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::CONFIRM_DELETE_REGISTER'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.deleteError({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public deleteError({ component }: IEventParam<Button>) {
    DialogService.hide();
    LoadingService.show(' ');
    const item = localStorage.getItem('request-error-log') || '[]';
    const errorLogs: RequestErrorLog[] = JSON.parse(item);
    const timestamp = component.name.split('-')[1] || '';
    const index = errorLogs.findIndex((el: IDictionary) => `${el.timestamp}` === timestamp);
    if (index !== -1) {
      // const { date } = errorLogs[index];
      errorLogs.splice(index, 1);
      const stringified = JSON.stringify(errorLogs);
      localStorage.setItem('request-error-log', stringified);
      this.mountErrors();
    }
    LoadingService.hide();
  }

  public clickErrorCard({ component }: IEventParam<Card>) {
    const item = localStorage.getItem('request-error-log') || '[]';
    const errorLogs: RequestErrorLog[] = JSON.parse(item);
    const timestamp = component.name.split('_')[1] || '';
    const index = errorLogs.findIndex((el: IDictionary) => `${el.timestamp}` === timestamp);
    const error = errorLogs[index];
    if (Object.entries(error.row).length > 0) {
      this.showDetailError = true;
      this.errorDescription = `${error.timestamp} - ${error.objectName}`;
      this.errorDetail = `${error.timestamp}`;
      this.errorDataRow = error.row;
    }
  }

  public get tabsVisibility() {
    return !this.showDetailError;
  }

  public clickBackDetailError() {
    this.showDetailError = false;
  }

  public async markErrorsSeen() {
    const item = localStorage.getItem('request-error-log') || '[]';
    const errorLogs: RequestErrorLog[] = JSON.parse(item);
    await errorLogs.forEach((el: IDictionary) => {
      el.seen = true;
    });
    const stringified = JSON.stringify(errorLogs);
    localStorage.setItem('request-error-log', stringified);
    const comp = Metadata.getInstance<Badge>('badgeErrorNotifications');
    this.setErrorsBadgeCount({ component: comp });
  }

  public translateOperation({ component }: IEventParam<Text>) {
    const text = component.name.split('_')[2];
    component.text = `${I18n.translate('Operação')}: ${I18n.translate(text)}`;
  }

  public setErrorsBadgeCount({ component }: IEventParam<Badge>) {
    const errors = JSON.parse(localStorage.getItem('request-error-log') || '[]');
    component.badge = errors.filter((el: IDictionary) => el.seen === false).length;
  }

  public setFormattedCode({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0];
    const code = value.split('_')[0];
    component.text = `${I18n.translate('Código')}: ${this.utilities.getValueWithMaskProduto(code)}`;
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0];
    const qt = value.split('_')[1];
    const mask = Config.masks.productMask as ScmMask;
    const maskNumber = this.utilities.getMaskNumber(qt, mask);
    component.text = `${I18n.translate('Qtde')}: ${maskNumber}`;
  }
}
